import random
import math
random.seed("personal02/maxnumber")

class TestGen:
    def __init__(self):
        self.test_number = 0
        self.subtask = ""
        self.desc_file = open("../tests.desc", "w")

    def generate_samples(self):             
        self.print_test([1, 3, 3, 6], "sample 1", 0, "preliminary")
        self.print_test([2, 2, 2, 2, 4, 4], "sample 2", 0, "preliminary")

    def generate_random_test(self, maxn, maxx):
        a = []
        n = maxn
        for i in range(n):
            x = random.randint(1, maxx // 2)
            exponent = 1
            while x * 2**exponent <= maxx:
                exponent += 1
            exponent -= 1
            a.append(x * 2**random.randint(0, exponent))
        random.shuffle(a)
        self.print_test(a, "random test, n = %d" % n)

    def generate_random_power_of_two_test(self, maxn):
        a = []
        n = maxn
        for i in range(n):
            x = random.randint(1, 20)
            a.append(2**x)
        random.shuffle(a)
        self.print_test(a, "random test with powers of 2, n = %d" % n)

    def generate_first_group_of_tests(self, folder):
        self.subtask = folder
        self.print_test([1], "min test, elements = [1]")
        self.print_test([2], "min test, elements = [2]")
        self.print_test([2**20], "min test, elements = [2^30]")
        self.print_test([2**20, 2**20], "min test, elements = [2^30, 2^30]")
        self.print_test([2, 2, 4, 8, 1, 1], "min test, n = 6")
        maxn = 1000
        maxx = 2**20
        ns = [5, 10, 100, maxn]
        for n in ns:
            self.generate_random_power_of_two_test(n)
        for it in range(5):
            b = []
            for i in range(maxn):
                b.append(2**random.randint(0, 20))
            random.shuffle(b)
            self.print_test(b, "random max test with many equal numbers, n = %d" % len(b))

    def generate_max_tests(self, maxn, maxx):
        a = [maxx] * maxn
        self.print_test(a, "max test, all elements equal to %d, n = %d" % (maxx, maxn))
        for it in range(10):
            bound = random.randint(49, 149)
            b = []
            for i in range(maxn):
                b.append(random.randint(1, bound))
                left = maxx // bound
                power = 1
                exponent = 0
                while power <= left:
                    power *= 2
                    exponent += 1
                exponent -= 1
                power //= 2
                b[i] *= 2**random.randint(0, exponent)
            random.shuffle(b)
            self.print_test(b, "random max test with many equal numbers, n = %d" % len(b))
    
    def generate_second_group_of_tests(self, folder):
        self.subtask = folder
        self.print_test([1], "min test, elements = [1]")
        self.print_test([2], "min test, elements = [2]")
        self.print_test([2000000000], "min test, elements = [2000000000]")
        self.print_test([2**30, 2**30], "min test, elements = [2^30, 2^30]")
        self.print_test([2, 2, 4, 8, 1, 1, 15], "min test, n = 7")
        self.print_test([2, 4, 8, 1, 1, 15, 15, 7, 7, 7, 7, 7, 7, 7], "min test, n = 14")
        self.print_test([2, 7, 4, 8, 1, 1, 15, 15, 7, 7, 7, 7, 7, 7, 7], "min test, n = 15")
        self.print_test([1, 2, 5, 6, 3], "min test, n = 5")
        maxn = 10**5
        maxx = 2 * 10**9
        ns = [3, 5, 10, 100, 1000, 10**4, maxn]
        nx = [10, 100, 10000, 65535, 10**6, maxx]
        for n in ns:
            for x in nx:
                self.generate_random_test(n, x)
        self.generate_max_tests(maxn, maxx)
                       
    def print_test(self, a, description, points=0, folder="tests"):
        self.test_number += 1
        print("Generating test %d" % self.test_number)
        
        dirname = os.path.join('..', folder, self.subtask)
        test_name = os.path.join(dirname, "{0:0=2d}".format(self.test_number))
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        test_file = open(test_name, "w")
        print(len(a), file=test_file)
        print(' '.join(str(element) for element in a), file=test_file)

        print(test_name + ", folder=" + folder + ", subtask=" + self.subtask + ", points=" + str(points) + ", comment=\"" + description + "\"", file=self.desc_file)
        test_file.close()

    def generate_all_tests(self):
        self.generate_samples() 
        self.test_number = 0
        
        #First group of tests
        self.generate_first_group_of_tests("subtask1")
        print("comment_subtask1=\"n &le; 1000, elements are power of 2 and &le; 2^20\"", file=self.desc_file)
        print("bonus_subtask1=40", file=self.desc_file)
        
        #Second group of tests
        self.generate_second_group_of_tests("subtask2")
        print("comment_subtask2=\"n &le; 100000\"", file=self.desc_file)
        print("bonus_subtask2=60", file=self.desc_file)

        self.desc_file.close()

import os, shutil
if os.path.exists('../tests'):
    shutil.rmtree('../tests', ignore_errors=False, onerror=None)
if os.path.exists('../preliminary'):
    shutil.rmtree('../preliminary', ignore_errors=False, onerror=None)
try:
    os.makedirs('../tests')
    os.makedirs('../preliminary')
except OSError:
    pass

import time
start = time.time()
writer = TestGen()
writer.generate_all_tests()
finish = time.time()
print('All tests were generated. Elapsed time: ', finish - start)
